// Canvases
var vibf = document.getElementById("viberation_figure"),
    convibf = vibf.getContext("2d"),
    omef = document.getElementById("omega_figure"),
    conomef = omef.getContext("2d");

// Get Elements
var massRange = document.getElementById("massRange"),
    waveRange = document.getElementById("waveRange"),
    typeButton = document.getElementById("typeButton"),
    startButton = document.getElementById("startButton");
mass = document.getElementById("mass");
wave = document.getElementById("wave");

// Plot Variables
var st = 40;                        // interval in vibf
var ST = 2;
var x_arr = new Array(14),
    y_arr = new Array(14);

// Physical Variables
var N = 4000;                       // cell numbers
var type = true,                    // true = Accounstics, false = Optics
    paused = true;
var A = 100, B = 100;               // amplitude of P & Q
var x = 0, y = 0, X = 0, Y = 0;     // coordinates of vibf & omef
var t = 0;                          // time
var h = 0, q = 0;                   // wavenumber
var r = 5;                          // radius of P & Q
var omega = 1;                      // angular frequency
var M = 1;                          // mass of Q

// Functions
function updateInfo() {
    h = waveRange.value;
    M = massRange.value;
    type = typeButton.value;
    paused = startButton.value;
    q = Math.PI * Number(h) / (Number(N));
    drawOmef();
}

function drawOmef() {
    conomef.clearRect(0, 0, omef.width, omef.height);
    conomef.save();

    conomef.beginPath();            // x axis
    conomef.moveTo(0, 300);
    conomef.lineTo(300, 300);
    convibf.lineWidth = 0.5;
    convibf.strokeStyle = "grey";
    conomef.stroke();

    conomef.beginPath();            // y axis
    conomef.moveTo(150, 300);
    conomef.lineTo(150, 0);
    convibf.lineWidth = 0.5;
    convibf.strokeStyle = "grey";
    conomef.stroke();

    conomef.beginPath();            // accounstics
    for (var k = 0; k <= 300; k++) {
        X = cX(Number(k));
        Y = Math.sqrt((Number(M) + 1) / Number(M) * (1 - Math.sqrt(1 - 4 * Number(M) / Math.pow((Number(M) + 1), 2) * Math.pow(Math.sin(Number(X)), 2))));
        if (k == 0) {
            conomef.moveTo(k, iY(Y));
        } else {
            conomef.lineTo(k, iY(Y));
        }
        convibf.lineWidth = 0.5;
        convibf.strokeStyle = "black";
        conomef.stroke();
    }

    conomef.beginPath();            // optics
    for (var k = 0; k <= 300; k++) {
        X = cX(Number(k));
        Y = Math.sqrt((Number(M) + 1) / Number(M) * (1 + Math.sqrt(1 - 4 * Number(M) / Math.pow((Number(M) + 1), 2) * Math.pow(Math.sin(Number(X)), 2))));
        if (k == 0) {
            conomef.moveTo(k, iY(Y));
        } else {
            conomef.lineTo(k, iY(Y));
        }
        convibf.lineWidth = 0.5;
        convibf.strokeStyle = "black";
        conomef.stroke();
    }

    conomef.beginPath();            // current point
    X = (Number(q) + Math.PI / 2) * 300 / Math.PI;
    if (type) {
        Y = iY(Math.sqrt((Number(M) + 1) / Number(M) * (1 - Math.sqrt(1 - 4 * Number(M) / Math.pow((Number(M) + 1), 2) * Math.pow(Math.sin(Number(q)), 2)))));
    } else {
        Y = iY(Math.sqrt((Number(M) + 1) / Number(M) * (1 + Math.sqrt(1 - 4 * Number(M) / Math.pow((Number(M) + 1), 2) * Math.pow(Math.sin(Number(q)), 2)))));
    }
    conomef.arc(X, Y, 7, 0, 2 * Math.PI);
    conomef.fillStyle = "orange";
    conomef.fill();
    conomef.lineWidth = 0.5;
    conomef.stroke();

    conomef.restore();

    conomef.fillText("q", 200, 313);
    conomef.fillText("ω", 160, 15);
    conomef.fillText("ω+", 200, 40);
    conomef.fillText("ω-", 200, 270);
    conomef.fillText("+π/2a", 250, 313);
    conomef.fillText("-π/2a", 10, 313);
}

function drawVibf() {
    convibf.save();

    convibf.beginPath();
    convibf.moveTo(40, 150);
    convibf.lineTo(560, 150);
    convibf.lineWidth = 1;
    convibf.strokeStyle = "black";
    convibf.stroke();
    // draw equilibrium line

    if (type) {
        omega = Math.sqrt((Number(M) + 1) / Number(M) * (1 - Math.sqrt(1 - 4 * Number(M) / Math.pow((Number(M) + 1), 2) * Math.pow(Math.sin(Number(q)), 2))));
    } else {
        omega = Math.sqrt((Number(M) + 1) / Number(M) * (1 + Math.sqrt(1 - 4 * Number(M) / Math.pow((Number(M) + 1), 2) * Math.pow(Math.sin(Number(q)), 2))));
    }
    B = -A * (Math.pow(Number(omega), 2) - 2) / (2 * Math.cos(Number(q)));
    // calculate angular frequency & amplitude

    for (var k = 0; k <= 6; k++) {
        x = st + st * (2 * k);
        y = 150 - A * Math.cos(omega * t - q * (x - st) / 40);
        convibf.beginPath();
        convibf.arc(x, y, r, 0, 2 * Math.PI);
        convibf.fillStyle = "red";
        convibf.fill();
        convibf.lineWidth = 0.5;
        convibf.stroke();
        x_arr[2 * k] = x;
        y_arr[2 * k] = y;
    }   // draw Ps

    for (var k = 0; k <= 6; k++) {
        x = st + st * (2 * k + 1);
        y = 150 - B * Math.cos(omega * t - q * (x - st) / 40);
        convibf.beginPath();
        convibf.arc(x, y, r, 0, 2 * Math.PI);
        convibf.fillStyle = "green";
        convibf.fill();
        convibf.lineWidth = 0.5;
        convibf.stroke();
        x_arr[2 * k + 1] = x;
        y_arr[2 * k + 1] = y;
    }   //draw Qs

    for (var k = 0; k <= 12; k++) {
        if (k % 2 == 0) {
            x = (x_arr[k] + M * x_arr[k + 1]) / (1 + Number(M));
            y = (y_arr[k] + M * y_arr[k + 1]) / (1 + Number(M));
        } else {
            x = (M * x_arr[k] + x_arr[k + 1]) / (1 + Number(M));
            y = (M * y_arr[k] + y_arr[k + 1]) / (1 + Number(M));
        }
        convibf.beginPath();
        convibf.arc(x, y, r / 2, 0, 2 * Math.PI);
        convibf.fillStyle = "yellow";
        convibf.fill();
        convibf.lineWidth = 0.5;
        convibf.stroke();
    }   // draw mass center

    convibf.restore();
}

function draw(currentTime) {
    requestAnimFrame(draw);
    if (!paused) {
        if (type) {
            t = 0.01 * currentTime;
        } else {
            t = 0.001 * currentTime;
        }
    }
    convibf.clearRect(0, 0, vibf.width, vibf.height);
    drawVibf();
}

function cX(x) {
    return Math.PI / 300 * Number(x) - Math.PI / 2;
}   // x: px -> real

function iY(Y) {
    return -150 * Y + 300;
}   // y: real -> px


// Events
waveRange.onchange = function (e) {
    h = waveRange.value;
    q = Math.PI * Number(h) / (Number(N));
    wave.innerHTML = q.toFixed(3) + "/a";
    drawOmef();
}


massRange.onchange = function (e) {
    M = massRange.value;
    mass.innerHTML = M;
    drawOmef();
}

startButton.onclick = function (e) {
    if (paused) {
        paused = false;
        startButton.value = "停止";
    }
    else {
        paused = true;
        startButton.value = "开始";
        updateInfo();
    }
}

typeButton.onclick = function (e) {
    if (type) {
        type = false;
        typeButton.value = "光学波";
        drawOmef();
    }
    else {
        type = true;
        typeButton.value = "声学波";
        drawOmef();
    }
}

// Initialization
conomef.font = "14px Arial";
conomef.fillStyle = "black";
updateInfo();
draw();